@@ -349,7 +349,13 @@ attr_get_param(int attr)
 	int i;
 	AttrStack *stack = stack_of_stacks_top;
 	if (!stack) {
-		warning_handler("No stack to get attribute from");
+		if (attr != ATTR_ENCODING) {
+			/*
+			 * attr_get_param(ATTR_ENCODING) is always called
+			 * called once without a stack being available.
+			 */
+			warning_handler("No stack to get attribute from");
+		}
 		return NULL;
 	}
 
@@ -443,13 +449,10 @@ void
 attrstack_push () 
 {
 	AttrStack *new_stack;
-	AttrStack *prev_stack;
 
 	new_stack = (AttrStack*) my_malloc (sizeof (AttrStack));
 	memset ((void*) new_stack, 0, sizeof (AttrStack));
 
-	prev_stack = stack_of_stacks_top;
-
 	if (!stack_of_stacks) {
 		stack_of_stacks = new_stack;
 	} else {
@@ -572,7 +575,12 @@ attrstack_drop ()
 	while(prev_stack && prev_stack->next && prev_stack->next != stack)
 		prev_stack = prev_stack->next;
 
-	if (prev_stack) {
+        /* prev_stack == stack_of_stacks_top if there was only one
+           element. We'll be empty. Note that prev_stack can't be
+           actually NULL here, this would happen with an empty stack,
+           which would have been detected by the !stack test above.
+        */
+	if (prev_stack && (prev_stack != stack_of_stacks_top)) {
 		stack_of_stacks_top = prev_stack;
 		prev_stack->next = NULL;
 	} else {
@@ -687,7 +695,7 @@ safe_printf(int nr, char *string, ...)
 	va_list arguments;
 
 	if (string == NULL)
-		printf("");
+		;
 	else
 	{
 		va_start(arguments, string);
@@ -787,7 +795,7 @@ assemble_string(char *string, int nr)
  *=======================================================================*/
 
 Collection *
-add_to_collection(Collection *col, int nr, char *text)
+add_to_collection(Collection *col, int nr, const char *text)
 {
 	Collection *c = col;
 
@@ -832,7 +840,7 @@ add_to_collection(Collection *col, int nr, char *text)
  * Returns:	NULL or found elemet.
  *=======================================================================*/
 
-char *
+const char *
 get_from_collection(Collection *c, int nr)
 {
 	while (c != NULL)
