@@ -24,8 +24,7 @@
  * Module name:    hash
  * Author name:    Zachary Smith
  * Create date:    01 Sep 00
- * Purpose:        Word-hash management. Words are put into a hash and an
- *                 identifier is returned. This is used to save us from
+ * Purpose:        shared string storage. This is used to save us from
  *                 doing multiple mallocs for recurring strings such as
  *                 'the' and \par. This is not a big issue under Unix,
  *                 but it is under other OSes and anyway, waste not want not.
@@ -39,6 +38,7 @@
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requsted by ZT Smith
  * 06 Jan 06, marcossamaral@terra.com.br: changes hash_stats function
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
+ * Further: see VCS log
  *--------------------------------------------------------------------*/
 
 #ifdef HAVE_CONFIG_H
@@ -53,6 +53,10 @@
 #include <string.h>
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #include "error.h"
 #include "main.h"
 #include "malloc.h"
@@ -60,36 +64,12 @@
 
 typedef struct _hi {
 	struct _hi *next;
-	char *str;
-	unsigned long value;
+	char str[1];
 } HashItem;
 
 
-/* Index by first char of string */
+/* Index by first char of string not \ */
 static HashItem *hash[256];
-static unsigned long hash_length[256];
-static unsigned long hash_value=0;
-
-
-
-/*========================================================================
- * Name:	hash_init
- * Purpose:	Clear the hash table.
- * Args:	None.
- * Returns:	None.
- *=======================================================================*/
-
-void
-hash_init ()
-{
-	int i;
-	for (i=0; i<256; i++) {
-		hash[i]=NULL;
-		hash_length[i]=0;
-	}
-}
-
-
 
 /*========================================================================
  * Name:	hash_stats
@@ -105,10 +85,15 @@ hash_stats ()
 {
 	int i;
 	unsigned long total=0;
+	HashItem *hi;
 	for (i=0; i<256; i++) {
-		total += hash_length[i];
+		hi = hash[i];
+		while (hi) {
+			total++;
+			hi = hi->next;
+		}
 	}
-	return(total);
+	return total;
 }
 
 
@@ -124,95 +109,46 @@ static HashItem *
 hashitem_new (char *str)
 {
 	HashItem *hi;
-	unsigned long i;
 
-	hi=(HashItem*) my_malloc(sizeof(HashItem));
+        /* We already have a char[1], so allocating strlen() does
+           yield space for the ending 0 */
+        unsigned int l = sizeof(HashItem)+strlen(str);
+	hi=(HashItem*) my_malloc(l);
 	if (!hi)
 		error_handler("Out of memory");
-	memset ((void*)hi, 0, sizeof (HashItem));
-
-	hi->str = my_strdup(str);
-
-	i = (unsigned char)*str;
-	if (i=='\\') i=(unsigned char)str[1];
-	i <<= 24;
-	hi->value = i | (hash_value++ & 0xffffff);
+	strcpy(hi->str, str);
 	hi->next = NULL;
 
-#if 0
-	if (debug_mode) {
-		printf ("<!-- storing val %08lx str %s -->\n",
-			hi->value, hi->str);
-	}
-#endif
-
 	return hi;
 }
 
 
 /*========================================================================
- * Name:	hash_get_index
- * Purpose:	Given a string, returns the "index" i.e. the word identifier.
+ * Name:	hash_store
+ * Purpose:	Given a string, find or store it and return the address of
+ *              the copy.
  * Args:	String.
- * Returns:	Index.
+ * Returns:	Address of readonly clone.
  *=======================================================================*/
 
-unsigned long
-hash_get_index (char *str)
+const char *
+hash_store (char *str)
 {
-#if 1 /* daved - 0.19.1 */
-	unsigned short index;
-	unsigned char ch;
-#else
-	int index;
-	char ch;
-#endif
+	unsigned char ch = (unsigned char)*str;
 	HashItem *hi;
 
-#if 1 /* daved - 0.19.1 */
-	ch = (unsigned char)*str;
-#else
-	ch = *str;
-#endif
 	if (ch=='\\' && *(str+1))
 		ch = *(str+1); 
-	index = ch;
-	hi = hash[index];
+	hi = hash[ch];
 	while (hi) {
-		if (!strcmp(hi->str,str))
-			return hi->value;
-		hi=hi->next;
+		if (!strcmp(hi->str, str))
+			return hi->str;
+		hi = hi->next;
 	}
 	/* not in hash */
-	hi = hashitem_new (str);
-	hi->next = hash[index];
-	hash [index] = hi;
-	++hash_length [index];
-	return hi->value;
+	hi = hashitem_new(str);
+	hi->next = hash[ch];
+	hash[ch] = hi;
+	return hi->str;
 }
 
-
-/*========================================================================
- * Name:	hash_get_string
-
- * Purpose:	Given the index (word identifier) returns the word string.
- * Args:	Index.
- * Returns:	String, or NULL if not found.
- *=======================================================================*/
-
-char*
-hash_get_string (unsigned long value)
-{
-	int index;
-	HashItem *hi;
-
-	index = value >> 24;
-	hi = hash[index];
-	while (hi) {
-		if (hi->value == value)
-			return hi->str;
-		hi=hi->next;
-	}
-	warning_handler("Word not in hash");
-	return NULL;
-}
