@@ -218,7 +218,6 @@ main (int argc, char **argv)
 	}
 
 
-	hash_init();
 	
 	/* Program information */
 	if (verbose_mode || debug_mode) {
@@ -263,7 +262,7 @@ main (int argc, char **argv)
 		printf("\n");
 	} else {
 /* Should we also optimize word before dump? - AF */
-		word = optimize_word(word);
+		word = optimize_word(word, 1);
 		word_print(word);
 	}
 
