@@ -14,6 +14,6 @@ struct path_dir
 
 extern int   check_dirs();
 extern void  show_dirs();
-extern char *search_in_path(char *name, char *suffix);
+extern char *search_in_path(const char *name, char *suffix);
 
 #endif /* _PATH_H_INCLUDED_ */
